﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class GameLogicScript : MonoBehaviour 
{
	public bool doGuiUpdate = true;
	public bool isPaused = false;
	public bool isLevelEnded = false;
	public bool isGameOver = false;

	public GameObject marker;

	public Transform player;
	float playerMoveSpeed = 35.0f;
	float playerTurnSpeed = 5.0f;
	public Animator playerAnimator;

	public GameObject laserBolt;
	public Transform laserBoltSpawnPoint;
	public GameObject enemyLaserBolt;

	public GameObject enemyTank;
	public float enemySpawnRate = 1.0f;
	public int maxEnemySpawns = 32;
	int currentEnemySpawns = 0;
	float enemySpawnCounter = 0.0f;
	GameObject[] listOfEnemySpawnGameObjects;

	public GameObject explosionEffect;

	public float playerShootInterval = 0.5f;
	float playerLastShotTime = 0.0f;

	Transform mainCamera;
	Vector3 mainCameraOffset;

	GameObject persistentLogicAnchor = null;
	PersistentLogicScript persistentGameLogicScript = null;

	GameObject healthBarGameObject;
	Text scoreText;
	Text waveText;
	GameObject quitButton;
	Text pickupsText;

	RawImage wellDoneImage;
	RawImage pausedImage;
	RawImage gameOverImage;

	bool isValidMousePositionInWorld = false;
	Vector3 mousePositionInWorld = Vector3.zero;

	int nosCollectiblesInLevel;

	void Start ()
	{
		persistentLogicAnchor = GameObject.Find("PersistentLogicAnchor");
		if(persistentLogicAnchor == null)
		{
			Application.LoadLevel("Start");
			return;
		}
		persistentGameLogicScript = persistentLogicAnchor.GetComponent<PersistentLogicScript>();

		healthBarGameObject = GameObject.Find ("HealthBarFG");
		scoreText = GameObject.Find ("ScoreText").GetComponent<Text>();
		waveText = GameObject.Find ("WaveText").GetComponent<Text>();
		pickupsText = GameObject.Find ("PickupsText").GetComponent<Text>();

		mainCamera = GameObject.Find ("Main Camera").transform;
		mainCameraOffset = mainCamera.transform.localPosition;

		wellDoneImage = GameObject.Find ("WellDoneImage").GetComponent<RawImage>();
		pausedImage = GameObject.Find ("PausedImage").GetComponent<RawImage>();
		gameOverImage = GameObject.Find ("GameOverImage").GetComponent<RawImage>();

		quitButton = GameObject.Find ("QuitButton");

		//Cursor.visible = false;

		playerLastShotTime = playerShootInterval;

		PrepareCollectables();
		PrepareEnemySpawns();
	}

	void Update ()
	{
		if(persistentGameLogicScript == null)
		{
			return;
		}
		
		if(isGameOver)
		{
			return;
		}
		else
		{			
			if(persistentGameLogicScript.healthPercentage <= 0.0f)
			{
				isGameOver = true;
				StartCoroutine(FinishGame());
				return;
			}
		}

		if(isLevelEnded)
		{
			return;
		}
		else
		{			
			if(nosCollectiblesInLevel == 0)
			{
				isLevelEnded = true;
				StartCoroutine(FinishLevel());
				return;
			}
		}

		if(!isPaused)
		{
			UpdateMousePositionInWorld();

			if(Input.GetButtonUp("Cancel"))
			{
				TogglePausedState();
			}

			/*if(Input.GetButtonUp("Fire3"))
			{
				persistentGameLogicScript.WarpToNextLevel();
				return;
			}*/

			if(Input.GetButton("Fire1"))
			{
				HandlePlayerShooting(Time.deltaTime);
			}

			HandlePlayerMovement();
			HandleCameraFollowingPlayer();

			HandleRandomEnemySpawn(Time.deltaTime);

			if(doGuiUpdate)
			{
				UpdateHealthPercentage();
				UpdateScore();
				UpdateWave();
				UpdateCollected();

				doGuiUpdate = false;
			}
		}
		else
		{
			if(Input.GetButtonUp("Cancel"))
			{
				TogglePausedState();
			}
		}
	}

	void TogglePausedState()
	{
		isPaused = !isPaused;
		pausedImage.enabled = isPaused;

		quitButton.GetComponent<Image>().enabled = isPaused;
		quitButton.GetComponent<Button>().interactable = isPaused;
	}

	public void OnClickQuitButton ()
	{
		persistentGameLogicScript.LoadLevelByName ("Titles");
	}

	void UpdateMousePositionInWorld()
	{
		isValidMousePositionInWorld = CalculateMouseClickPositionInWorld(out mousePositionInWorld);
	}

	void HandlePlayerMovement()
	{
		float horizontalInputValue = Input.GetAxis("Horizontal");
		float verticalInputValue = Input.GetAxis("Vertical");

		Vector3 lookAtPosition = mousePositionInWorld - player.transform.position;
		lookAtPosition.y = 0.0f;
		float step = playerTurnSpeed * Time.deltaTime;
		Vector3 newDir = Vector3.RotateTowards(player.transform.forward, lookAtPosition, step, 0.0F);
		player.transform.rotation = Quaternion.LookRotation(newDir);

		Vector3 cameraForward = Camera.main.transform.forward;
		cameraForward.y = 0.0f;
		cameraForward.Normalize();
		Vector3 cameraRight = Camera.main.transform.right;
		cameraRight.y = 0.0f;
		cameraRight.Normalize();

		Vector3 sumMovement = (cameraRight * horizontalInputValue) + (cameraForward * verticalInputValue);
		sumMovement.Normalize();
		player.transform.position += sumMovement * playerMoveSpeed * Time.deltaTime;

		if(Vector3.Magnitude(sumMovement) > 0.25f)
		{
			playerAnimator.Play("walk");
		}
		else
		{
			playerAnimator.Play("idle");
		}
	}
	
	void HandleCameraFollowingPlayer()
	{
		mainCamera.position = player.position + mainCameraOffset;
	}

	void HandlePlayerShooting(float deltaTime)
	{
		playerLastShotTime += deltaTime;

		if(isValidMousePositionInWorld && playerLastShotTime >= playerShootInterval)
		{
			Vector3 direction = mousePositionInWorld - laserBoltSpawnPoint.position;
			Instantiate(laserBolt, laserBoltSpawnPoint.position, Quaternion.LookRotation(direction));
			
			playerLastShotTime -= playerShootInterval;
		}
	}

	public void CreateEnemyBolt(Vector3 position, Vector3 direction)
	{
		Instantiate(enemyLaserBolt, position, Quaternion.LookRotation(direction));
	}

	void UpdateHealthPercentage()
	{
		float newWidth = persistentGameLogicScript.healthPercentage * 2.56f; 
		healthBarGameObject.GetComponent<RectTransform>().SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, newWidth);
	}
	
	void UpdateScore()
	{
		scoreText.text = persistentGameLogicScript.score.ToString();
	}

	void UpdateWave()
	{
		waveText.text = "Wave: " + (persistentGameLogicScript.waveNr + 1).ToString();
	}

	void PrepareCollectables()
	{
		nosCollectiblesInLevel = GameObject.FindGameObjectsWithTag("Collectable").Length;
	}

	void PrepareEnemySpawns()
	{
		listOfEnemySpawnGameObjects = GameObject.FindGameObjectsWithTag("EnemySpawn");

		for(int index = 0; index < listOfEnemySpawnGameObjects.Length; index++)
		{
			for(int childIndex = 0; childIndex < listOfEnemySpawnGameObjects[index].transform.childCount; childIndex++)
			{
				listOfEnemySpawnGameObjects[index].transform.GetChild(childIndex).gameObject.SetActive(false);
			}
		}
	}

	void HandleRandomEnemySpawn(float deltaTime)
	{
		if(currentEnemySpawns > maxEnemySpawns)
		{
			return;
		}

		enemySpawnCounter += deltaTime;
		while(enemySpawnCounter >= enemySpawnRate) 
		{
			int randomIndex = Random.Range(0, listOfEnemySpawnGameObjects.Length);
			Instantiate(enemyTank, listOfEnemySpawnGameObjects[randomIndex].transform.position, listOfEnemySpawnGameObjects[randomIndex].transform.rotation);

			enemySpawnCounter -= enemySpawnRate;

			currentEnemySpawns++;
		}
	}

	public void MarkEnemyTankDied(Transform transformAtDeath)
	{
		Instantiate(explosionEffect, transformAtDeath.position, transformAtDeath.rotation);

		currentEnemySpawns--;
	}

	public void MarkOneCollected(Transform transformAtDeath)
	{
		persistentGameLogicScript.ModifyScore(1);

		Instantiate(explosionEffect, transformAtDeath.position, transformAtDeath.rotation);

		nosCollectiblesInLevel -= 1;

		UpdateCollected();
	}

	public void MarkImpact(Transform transformAtDeath)
	{
		Instantiate(explosionEffect, transformAtDeath.position, transformAtDeath.rotation);
	}

	IEnumerator FinishLevel()
	{
		wellDoneImage.enabled = true;
		
		yield return new WaitForSeconds(2.0f);	
		
		persistentGameLogicScript.WarpToNextLevel();
	}

	IEnumerator FinishGame()
	{
		gameOverImage.enabled = true;

		Vector3 pos1 = player.position;
		pos1.y += 17.0f;
		Instantiate(explosionEffect, pos1, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		Vector3 pos2 = player.position;
		pos2.y += 18.0f;
		Instantiate(explosionEffect, pos2, player.rotation);
		yield return new WaitForSeconds(0.5f);

		Vector3 pos3 = player.position;
		pos3.y += 16.0f;
		Instantiate(explosionEffect, pos3, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		Vector3 pos4 = player.position;
		pos4.y += 17.0f;
		Instantiate(explosionEffect, pos4, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		Vector3 pos5 = player.position;
		pos5.y += 18.0f;
		Instantiate(explosionEffect, pos5, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		Vector3 pos6 = player.position;
		pos6.y += 17.0f;
		Instantiate(explosionEffect, pos6, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		Vector3 pos7 = player.position;
		pos7.y += 16.0f;
		Instantiate(explosionEffect, pos7, player.rotation);
		yield return new WaitForSeconds(0.5f);	

		persistentGameLogicScript.EvaluateLastGame();
		persistentGameLogicScript.RestartGame();
		persistentGameLogicScript.LoadLevelByName ("Titles");
	}

	public void UpdateCollected()
	{
		pickupsText.text = "Pickups: " + nosCollectiblesInLevel.ToString();
	}

	private bool CalculateMouseClickPositionInWorld(out Vector3 target)
	{
		target = Vector3.zero;

		Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);

		RaycastHit hit;
		if (Physics.Raycast(ray, out hit))
		{
			target = hit.point;
			return true;
		}

		return false;
	}

	public float GetCurrentWaveModifier()
	{
		if(persistentGameLogicScript == null)
		{
			return 1.0f;
		}

		return 1.0f + (persistentGameLogicScript.waveNr * 0.075f);
	}

	public void RegisterPlayerHitByEnemyBolt(Transform transformAtDeath)
	{
		Instantiate(explosionEffect, transformAtDeath.position, transformAtDeath.rotation);

		persistentGameLogicScript.ModifyHealth(-15.0f);
	}
}
