﻿using UnityEngine;
using System.Collections;

public class HandleEnemyTank : MonoBehaviour 
{
	Transform player;
	public Transform turret;
	public Transform turretShotSpawn;

	public float navUpdateInterval = 2.0f;
	float timeSinceLastUpdate = 0.0f;

	GameObject gameLogicAnchor = null;
	GameLogicScript gameLogicScript = null;

	bool isMoving = true;

	public float shootInterval = 1.0f;
	public float shootDistance = 25.0f;
	float lastShotTime = 0.0f;

	void Start ()
	{
		gameLogicAnchor = GameObject.Find("GameLogicAnchor");
		if(gameLogicAnchor == null)
		{
			return;
		}
		gameLogicScript = gameLogicAnchor.GetComponent<GameLogicScript>();

		GameObject playerGameObject = GameObject.Find ("PlayerMovementRoot");
		player = playerGameObject.GetComponent<Transform>();

		timeSinceLastUpdate = navUpdateInterval;

		lastShotTime = shootInterval;
	}

	void Update ()
	{
		if(gameLogicScript == null)
		{
			return;
		}

		if(gameLogicScript.isLevelEnded || gameLogicScript.isGameOver)
		{
			if(isMoving)
			{
				GetComponent<NavMeshAgent>().Stop();
				isMoving = false;
			}
			
			return;
		}

		if(gameLogicScript.isPaused)
		{
			if(isMoving)
			{
				GetComponent<NavMeshAgent>().Stop();
				isMoving = false;
			}

			return;
		}
		else
		{
			if(!isMoving)
			{
				GetComponent<NavMeshAgent>().Resume();
				isMoving = true;
			}
		}

		timeSinceLastUpdate += Time.deltaTime;
		if(timeSinceLastUpdate >= navUpdateInterval)
		{
			UpdatePlayerTarget();

			timeSinceLastUpdate -= navUpdateInterval;
		}

		HandleShooting(Time.deltaTime);
	}

	void HandleShooting(float deltaTime)
	{
		float modifier = gameLogicScript.GetCurrentWaveModifier();

		Vector3 vectorToPlayer = player.position - transform.position;
		vectorToPlayer.y += 6.0f;

		turret.rotation = Quaternion.LookRotation(vectorToPlayer);

		lastShotTime += deltaTime;
		if(lastShotTime >= (shootInterval / modifier))
		{
			float distanceToPlayer = Vector3.Magnitude(vectorToPlayer);

			if(distanceToPlayer <= (shootDistance * modifier))
			{
				gameLogicScript.CreateEnemyBolt(turretShotSpawn.transform.position, vectorToPlayer);
				lastShotTime = 0.0f;
			}
		}
	}

	void OnTriggerEnter(Collider other) 
	{
		if(other.gameObject.tag != "Bolt")
		{
			return;
		}

		gameLogicScript.MarkEnemyTankDied(transform);

		Destroy (this.gameObject);
	}

	void UpdatePlayerTarget()
	{	
		GetComponent<NavMeshAgent>().destination = player.position;
	}
}
