﻿using UnityEngine;
using System.Collections;

public class PersistentLogicScript : MonoBehaviour 
{
	public int waveNr;
	public int score;
	public int hiscore;
	public float healthPercentage;

	public string[] LevelVariantNames;

	void Awake () 
	{
		GameObject.DontDestroyOnLoad(this.gameObject);

		LoadGame();

		Application.LoadLevel("Titles");
	}

	void Update ()
	{
	}

	public void RestartGame ()
	{
		waveNr = 0;
		score = 0;
		healthPercentage = 100.0f;
		
		SaveGame ();
	}

	public void EvaluateLastGame ()
	{
		if(score > hiscore)
		{
			hiscore = score;
		}
	}

	public void LoadGame ()
	{
		waveNr = PlayerPrefs.GetInt("waveNr", 0); 
		score = PlayerPrefs.GetInt("score", 0); 
		hiscore = PlayerPrefs.GetInt("hiscore", 0); 
		healthPercentage = PlayerPrefs.GetFloat("healthPercentage", 100.0f); 
	}

	public void SaveGame ()
	{
		if(score > hiscore)
		{
			hiscore = score;
		}

		PlayerPrefs.SetInt("waveNr", waveNr); 
		PlayerPrefs.SetInt("score", score); 
		PlayerPrefs.SetInt("hiscore", hiscore); 
		PlayerPrefs.SetFloat("healthPercentage", healthPercentage); 

		PlayerPrefs.Save ();
	}

	public void LoadLevelForWave()
	{
		int index = waveNr;	
		while(index >= LevelVariantNames.Length)
		{
			index -= LevelVariantNames.Length;
		}

		LoadLevelByName (LevelVariantNames[index]);
	}

	public void LoadLevelByName(string name)
	{
		Application.LoadLevel (name);	
	}

	public void ExitGame ()
	{
		Application.Quit ();
	}

	public void WarpToNextLevel()
	{
		waveNr += 1;
		if(waveNr > 9999)
		{
			waveNr = 0;
		}

		EvaluateLastGame();
		SaveGame ();

		LoadLevelForWave();
	}

	public void ModifyHealth(float value)
	{
		healthPercentage = Mathf.Clamp(healthPercentage + value, 0.0f, 100.0f);

		MarkUpdateUI();
	}

	public void ModifyScore(int value)
	{
		score += value;

		if(score > 9999999)
		{
			score = 0;
		}
		
		MarkUpdateUI();
	}

	public void MarkUpdateUI()
	{
		GameObject gameLogicAnchor = GameObject.Find ("GameLogicAnchor");
		if(gameLogicAnchor != null)
		{
			GameLogicScript gameLogicScript = gameLogicAnchor.GetComponent<GameLogicScript>();
			if(gameLogicScript != null)
			{
				gameLogicScript.doGuiUpdate = true;
			}
		}
	}
}
